<?php
	class Franchise_model extends CI_Model{

		public function __construct(){
			$this-> load-> database(); 
		}

		
//*****************inserting and updating methods*************************
        
        public function insert_franchise($image_name=NULL){
			$this->db->trans_start();
			$data = array(
				'name' => $this-> input-> post('name'),
				'description' => $this-> input-> post('description'),
				'phone_one'=> $this-> input-> post('phone_one'),
				'phone_two' => $this-> input-> post('phone_two'),
				'bank_account_number' => $this-> input-> post('bank_account_number'),
				'bank' => $this-> input-> post('bank'),
				'bank_account_name' => $this-> input-> post('bank_account_name'),
				'bank_account_branch' => $this-> input-> post('bank_account_branch')
			);

			if ($image_name!=NULL) {
				$data['logo'] = $this-> insert_franchise_logo($image_name,$data['name']);
			}

			//insert franchise
			$this-> db-> insert('franchise',$data);
			$franchise_id = $this->db->insert_id();

			$this-> update_franchise_shipping_zone_map($franchise_id);
			$this-> insert_franchise_search_code($franchise_id);
			
			$this->db->trans_complete();
			return $franchise_id;
		}

		public function update_franchise($franchise_id,$image_name=NULL){
			$this->db->trans_start();
			$data = array(
				'name' => $this-> input-> post('name'),
				'address' => $this-> input-> post('address'),
				'phone_one'=> $this-> input-> post('phone_one'),
				'phone_two' => $this-> input-> post('phone_two'),
				'bank_account_number' => $this-> input-> post('bank_account_number'),
				'bank' => $this-> input-> post('bank'),
				'bank_account_name' => $this-> input-> post('bank_account_name'),
				'bank_account_branch' => $this-> input-> post('bank_account_branch')
			);

			if ($image_name!=NULL) {
				$data['logo'] = $this-> insert_franchise_logo($image_name,$data['name']);
			}

			//update franchise
			$this-> db-> where('id',$franchise_id);
			$this-> db-> update('franchise',$data);

			$this-> update_franchise_shipping_zone_map($franchise_id);
			
			$this->db->trans_complete();
		}

		function insert_franchise_logo($image_name,$franchise_name){
			$data = array(
				'image_name' => $image_name,
				'image_alt' => $franchise_name
			);

			$this-> db-> insert('image',$data);
			return $this->db->insert_id();
		}
		
		function insert_franchise_search_code($franchise_id){
			$data = array(
			    'franchise' => $franchise_id,
				'order_prefix' => $this-> input-> post('order_prefix'),
				'last_order' => $this-> input-> post('last_order')
			);

			$this-> db-> insert('search_code',$data);
			return $this->db->insert_id();
		}

		function update_franchise_shipping_zone_map($franchise){
			//deleting data
			$this->db->delete('franchise_shipping_zone_map', array('franchise' => $franchise));

			//inserting data
			$shipping_zone = $this-> input-> post('shipping_zone');
			$franchise_shipping_cost = $this-> input-> post('franchise_shipping_cost');
			$franchise_charge_for_additional_kg = $this-> input-> post('franchise_charge_for_additional_kg');
			$service_charge = $this-> input-> post('service_charge'); 
			$charge_for_additional_kg = $this-> input-> post('charge_for_additional_kg'); 
			$data = array();
			foreach ($franchise_shipping_cost as $key => $item) {
				array_push($data, array('franchise' => $franchise,
										 'shipping_zone' => $shipping_zone[$key],
										 'franchise_shipping_cost' => $franchise_shipping_cost[$key],
										 'franchise_charge_for_additional_kg' => $franchise_charge_for_additional_kg[$key], 
										 'service_charge' => $service_charge[$key],
										 'charge_for_additional_kg'=> $charge_for_additional_kg[$key]));
			}
			$this->db->insert_batch('franchise_shipping_zone_map', $data); 	
		} 
		
		function update_franchise_fb_sponsor_amounts($franchise){

			$products = $this-> input-> post('products');
			$daily_sponsor_amount = $this-> input-> post('daily_sponsor_amount');
			
			$this->db->trans_start();
			foreach ($products as $key => $item) {
				$data = array(	
					'daily_sponsor_amount' => $daily_sponsor_amount[$key]
				);
				$this-> db-> where('franchise',$franchise);
				$this-> db-> where('product',$products[$key]);
				$this-> db-> update('franchise_product_map',$data);					
			}
			$this->db->trans_complete();
		} 
		
		function assign_franchise_products($franchise){
			//deleting data
			$this->db->delete('franchise_product_map', array('franchise' => $franchise));

			//inserting data
			$product = $this-> input-> post('product');
			$price = $this-> input-> post('price');
			$commision = $this-> input-> post('commision');
			$daily_sponsor_amount = $this-> input-> post('daily_sponsor_amount'); 
			
			$data = array();
			foreach ($product as $key => $item) {
				array_push($data, array('franchise' => $franchise,
										 'product' => $product[$key],
										 'price' => $price[$key],
										 'commision' => $commision[$key],
										 'daily_sponsor_amount' => $daily_sponsor_amount[$key]
										)
				);
			}
			$this->db->insert_batch('franchise_product_map', $data); 	
		} 


//*****************selecting methods**************************************

        public function get_franchise($id){
        	$this-> db-> select('franchise.id,franchise.name,franchise.description,franchise.phone_one,franchise.phone_two,franchise.address,franchise.bank,franchise.bank_account_number,franchise.bank_account_branch,franchise.bank_account_name,image.image_name as logo,image.image_alt as logo_alt');
        	$this-> db-> join('image', 'image.id = franchise.logo','left');
        	$query = $this-> db-> get_where('franchise',array('franchise.id'=> $id));
			return $query-> row();
        }

        public function get_franchise_shipping_zone_map($franchise){
        	$this-> db-> select('shipping_zone.id,shipping_zone.title,franchise_shipping_zone_map.service_charge,franchise_shipping_zone_map.charge_for_additional_kg,franchise_shipping_zone_map.franchise_shipping_cost,franchise_shipping_zone_map.franchise_charge_for_additional_kg');
        	$this-> db-> join('shipping_zone', 'shipping_zone.id = franchise_shipping_zone_map.shipping_zone');
        	$this-> db-> where('franchise_shipping_zone_map.franchise',$franchise);
        	$query = $this -> db -> get('franchise_shipping_zone_map');
			return $query->result_array();	
        }
        
        public function get_all_franchises(){
			$query = $this -> db -> get('franchise');
			return $query->result_array();	
		}
		
		public function get_all_sub_franchises(){
			$this-> db-> where('id<>','1'); //webstore
			$this-> db-> where('id<>','2'); //midnight buy
			$this-> db-> where('id<>','4'); //webstore online
			$this-> db-> where('id<>','18'); //gedaratama
			$query = $this -> db -> get('franchise');
			return $query->result_array();	
		}

		public function get_users_franchises($user){
			$this-> db-> select('franchise.name,user_franchise_map.franchise');
			$this-> db-> join('franchise', 'user_franchise_map.franchise = franchise.id');
			$query = $this -> db -> get_where('user_franchise_map',array('user'=> $user));
			return $query->result_array();			
		}
		
		public function get_users_franchise_ids($user){
			$this-> db-> select('user_franchise_map.franchise');
			$query = $this -> db -> get_where('user_franchise_map',array('user'=> $user));
			return array_column($query->result_array(), 'franchise');			
		}

		public function is_user_allowed_franchise($franchise, $user){
			$query = $this-> db-> get_where('user_franchise_map', array('user'=> $user, 'franchise' => $franchise));
			if (empty($query->row_array())) {
				return false;
			} else {
				return true;
			}	
		}

	}