<?php
	class Order_model extends CI_Model{

		public function __construct(){
			$this-> load-> database(); 
		}

//*****************inserting and updating methods*************************

		public function insert_order($search_code,$franchise, $user_id, $price_editable=FALSE){
		    
		    //checking whether search_code exists
		    $search_code_exists = $this-> Order_model-> get_order($search_code);
		    if(isset($search_code_exists)){
		        return $search_code;
		    }

			$this->db->trans_start();

			$data = array(
				'search_code' => $search_code,
				'franchise' => $franchise,
				'resipient'=> $this-> input-> post('resipient'),
				'private_reference'=> $this-> input-> post('private_reference'),
				'address'=> $this-> input-> post('address'),
				'city'=> $this-> input-> post('city_id'),
				'phone_one'=> $this-> input-> post('phone_one'), 
				'phone_two'=> $this-> input-> post('phone_two'),
				'payment_type'=> $this-> input-> post('payment_type'),
				'customer_notes'=> $this-> input-> post('customer_notes'),
				'delivery_notes'=> $this-> input-> post('delivery_notes'),
				'created_by'=> $user_id,
				'order_status'=> $this-> input-> post('order_status'),
				'last_modified'=> date('Y-m-d H:i:s')			
			);
			
			$this-> db-> insert('order',$data);
			$orderID = $this->db->insert_id();

			if ($data['order_status']=='hold') {
				$date = date_create_from_format('m/d/Y',$this-> input-> post('hold_until'));
				$hold_data = array(
					'order' => $orderID,
					'hold_until' => $date-> format('y-m-d')
				);
				$this-> db-> insert('hold_orders',$hold_data);
			}

			if ($data['order_status']=='not_confirm') {
				$date = new DateTime('tomorrow');
				$hold_data = array(
					'order' => $orderID,
					'hold_until' => $date-> format('y-m-d'),
					'reason' => $this-> input-> post('reason')
				);
				$this-> db-> insert('hold_orders',$hold_data);
			}
			
			//updating order modification history
			$this-> update_order_modification_history($orderID,$user_id,'Order created');
			
			//updating order table with summary
			$summary = $this-> insert_order_items($orderID,$franchise,$data['city'],$price_editable);
			$this-> db-> where('id',$orderID);
			$this-> db-> update('order',$summary);

			$this->db->trans_complete();

			return $search_code;
		}

		public function insert_order_items($order,$franchise,$city,$price_editable=FALSE){
			$summary =array('total_amount'=>0,'total_weight'=>0,'total_commision'=>0,'item_description'=>"");

			for ($i=1; $i < $this-> input-> post('RowCount'); $i++) { 

				if (!is_null($this-> input-> post('product_'.$i))) {
					if ((int)$this-> input-> post('qty_'.$i)>0) {
						//getting product data
						$product = $this-> Product_model-> get_franchises_products($franchise,$this-> input-> post('product_'.$i));
						
						//data for order_items table
						$data = array(
							'order'=> $order,
							'product'=> $this-> input-> post('product_'.$i),
							'quantity'=> $this-> input-> post('qty_'.$i),
							'amount'=> $product-> price * $this-> input-> post('qty_'.$i),
							'commision'=> $product-> commision * $this-> input-> post('qty_'.$i)			
						);

						if ($price_editable) {
							$data['amount'] = $this-> input-> post('amount_'.$i);
						}

						//data returned for order table
						$summary['total_amount']+= $data['amount'];
						$summary['total_weight']+= ($product-> weight * $data['quantity']);
						$summary['total_commision']+= $data['commision'];
						$summary['item_description'].= $product-> short_title."-".$data['quantity']."| ";

						//inserting data to order items 
						$this-> db-> insert('order_items',$data);
					}
				}
			}
			$summary['item_description'] = rtrim($summary['item_description'],"| ");
			$shipping_info = $this->Shipping_model-> get_shipping_info($franchise,$city);
			if ($price_editable && $this-> input-> post('edit_delivery')) {
				$summary['franchise_shipping_cost'] = $this-> input-> post('delivery_charge');
			}else{
				$summary['franchise_shipping_cost'] = $this->Shipping_model-> get_shipping_cost($summary['total_weight'],$shipping_info-> franchise_shipping_cost,$shipping_info-> franchise_charge_for_additional_kg);
			}
			$summary['service_charge'] = $this->Shipping_model-> get_shipping_cost($summary['total_weight'],$shipping_info-> service_charge,$shipping_info-> charge_for_additional_kg);
			$summary['grand_total'] = $summary['total_amount'] + $summary['franchise_shipping_cost'];
			return $summary;
		}

		public function update_order($order,$existing_order_status,$franchise, $user_id, $price_editable=FALSE,$waybill_editable=FALSE){

			$this->db->trans_start();

			$summary = $this-> update_order_items($order,$franchise,$this-> input-> post('city_id'),$price_editable);

			$data = array(
				'resipient'=> $this-> input-> post('resipient'),
				'private_reference'=> $this-> input-> post('private_reference'),
				'address'=> $this-> input-> post('address'),
				'city'=> $this-> input-> post('city_id'),
				'phone_one'=> $this-> input-> post('phone_one'), 
				'phone_two'=> $this-> input-> post('phone_two'),
				'payment_type'=> $this-> input-> post('payment_type'),
				'customer_notes'=> $this-> input-> post('customer_notes'),
				'delivery_notes'=> $this-> input-> post('delivery_notes'),
				'order_status'=> $this-> input-> post('order_status'),
				'total_amount' => $summary['total_amount'],
				'total_weight' => $summary['total_weight'],
				'total_commision' => $summary['total_commision'],
				'item_description' => $summary['item_description'],
				'franchise_shipping_cost' => $summary['franchise_shipping_cost'],
				'service_charge' => $summary['service_charge'],
				'grand_total' => $summary['grand_total'],
				'last_modified'=> date('Y-m-d H:i:s')
			);
			
			if($waybill_editable){
			    $data['courier'] = $this-> input-> post('courier');
			    $data['courier_reference'] = $this-> input-> post('courier_reference');
			}
			
			//10-02-2021 update
			if($data['order_status'] == 'confirm' && $existing_order_status != 'confirm'){
				$data['confirmed_by'] = $user_id;
			}

			$this-> db-> where('id',$order);
			$this-> db-> update('order',$data);

			if ($data['order_status']=='hold') {
				$this->db->delete('hold_orders', array('order' => $order));
				$date = date_create_from_format('m/d/Y',$this-> input-> post('hold_until'));
				$hold_data = array(
					'order' => $order,
					'hold_until' => $date-> format('y-m-d')
				);
				$this-> db-> insert('hold_orders',$hold_data);
			}

			if ($data['order_status']=='not_confirm') {
				$this->db->delete('hold_orders', array('order' => $order));
				$date = new DateTime('tomorrow');
				$hold_data = array(
					'order' => $order,
					'hold_until' => $date-> format('y-m-d'),
					'reason' => $this-> input-> post('reason')
				);
				$this-> db-> insert('hold_orders',$hold_data);
			}
		
			//10-02-2021 update
			//updating order modification history
			if($data['order_status'] == 'confirm' && $existing_order_status != 'confirm'){
				$this-> update_order_modification_history($order,$user_id,'Order status changed to '.$data['order_status'].' (call durarion:00:00:00)');
			}elseif($data['order_status'] == 'not_confirm' && $existing_order_status != 'not_confirm'){
				$this-> update_order_modification_history($order,$user_id,'Order status changed to '.$data['order_status'].' (call durarion:00:00:00)');
			}else{
				$this-> update_order_modification_history($order,$user_id,'Order was edited');
			}

			$this->db->trans_complete();
		}

		public function update_order_items($order,$franchise,$city,$price_editable=FALSE){
			$this->db->delete('order_items', array('order' => $order));
			return $this-> insert_order_items($order,$franchise,$city,$price_editable);
		}

		public function update_order_search_code($franchise){
			$query = $this-> db-> get_where('search_code',array('franchise'=> $franchise));
			$last_order = $query-> row()-> last_order + 1;
			$data = array(
				'last_order' => $last_order			
			);
			$this-> db-> where('franchise',$franchise);
			$this-> db-> update('search_code',$data);
		}

		public function update_order_status($order,$user_id,$status,$call_duration,$call_attempt=FALSE,$hold_until=FALSE,$reason=NULL){
			$this->db->trans_start();
			$data = array(
				'order_status'=> $status,
				'last_modified'=> date('Y-m-d H:i:s')			
			);
			if ($call_attempt) {
				$data['call_attempts'] = $this-> get_call_attempts($order) + 1;
			}
			
			//10-02-2021 update
			if($status =='confirm'){
				$data['confirmed_by'] = $user_id;
			}
			//end of update
			
			$this-> db-> where('id',$order);
			$result = $this-> db-> update('order',$data);
			$this->db->delete('hold_orders', array('order' => $order));

			if ($hold_until) {
				$hold_data = array(
					'order' => $order,
					'hold_until' => $hold_until,
					'reason' => $reason
				);
				$this-> db-> insert('hold_orders',$hold_data);
			}
			
			//updating order modification history
			$this-> update_order_modification_history($order,$user_id,'Order status changed to '.$status.' (call durarion: '.$call_duration.')');
			
			$this->db->trans_complete();
			return $result;
		}

		public function update_order_field($order,$user_id,$field, $value){
			//updates a single field in the order table
			$data = array(
				$field=> $value,
				'last_modified'=> date('Y-m-d H:i:s')			
			);
			$this-> db-> where('id',$order);
			$success = $this-> db-> update('order',$data);
			
			//updating order modification history
			$this-> update_order_modification_history($order,$user_id,$field.' was updated.');
			
			return $success;
		}
		
		public function update_order_modification_history($order,$user,$remarks){

		    $hold_data = array(
				'order' => $order,
				'user' => $user,
				'remarks' => $remarks
			);
			$this-> db-> insert('order_modification_history',$hold_data);
		}
		
		public function insert_additional_order_info($order_meta,$order_id){
			$this->db->trans_start();
			foreach ($order_meta as $meta) {
				$data = array(
					'order_meta' => $meta['meta_key'],
					'order_id' => $order_id,
					'value' => $this-> input-> post($meta['meta_key'])
				);
				$this-> db-> insert('additional_order_info',$data);
			}
			$this->db->trans_complete();	
		}

		public function update_additional_order_info($order_meta,$order_id){
			$this->db->trans_start();
			foreach ($order_meta as $meta) {
				$data = array(
					'value' => $this-> input-> post($meta['meta_key'])
				);
				$this-> db-> where('order_meta',$meta['meta_key']);
				$this-> db-> where('order_id',$order_id);
				$this-> db-> update('additional_order_info',$data);
			}
			$this->db->trans_complete();	
		}
		
		public function insert_compaint($order_id,$remark){
			$user_id = $this-> session-> userdata('user_id');
			$data = array(
				'order' => $order_id,
				'complaint_status'=> 'processing',
				'remark'=> $remark,
				'created_by'=> $user_id	
			);
			
			$this-> db-> insert('complaint',$data);
			return $this->db->insert_id();
		}

		public function update_compaint($complaint_id){
			$user_id = $this-> session-> userdata('user_id');
			$data = array(
				'checked_by' => $user_id,
				'complaint_status'=> 'sorted',
				'checked_date'=> date('Y-m-d H:i:s'),
				'solution'=> $this-> input-> post('solution')	
			);
			
			$this-> db-> where('id',$complaint_id);
			$this-> db-> update('complaint',$data);
		}
		
		public function cancel_packed_order($order_id,$update_stock=FALSE){
			//data for order_items table
			$order_data = array(
				'order_status'=> 'cancelled'			
			);

			$this-> db-> where('id',$order_id);
			$this-> db-> update('order',$order_data);
			
			if($update_stock){
			    $this-> Inventory_model-> returned_stock_by_order($order_id);   
			}
			
			//updating order modification history
			$user_id = $this-> session-> userdata('user_id');
			$this-> update_order_modification_history($order_id,$user_id,'Order status was changed to cancelled');
		}
		
		public function mark_as_damaged($order_id){
			//data for order_items table
			$order_data = array(
				'damaged_return'=> TRUE			
			);

			$this-> db-> where('id',$order_id);
			$this-> db-> update('order',$order_data);
			
			//updating order modification history
			$user_id = $this-> session-> userdata('user_id');
			$this-> update_order_modification_history($order_id,$user_id,'Order was marked as damaged');
		}
		
		public function mark_as_urgent($order_id){
			//data for order_items table
			$order_data = array(
				'is_urgent'=> TRUE			
			);

			$this-> db-> where('id',$order_id);
			$this-> db-> update('order',$order_data);
			
			//updating order modification history
			$user_id = $this-> session-> userdata('user_id');
			$this-> update_order_modification_history($order_id,$user_id,'Order was marked as urgent');
		}
		
		public function insert_bulk_orders($franchise,$user_id){
			$this->db->trans_start();

			$Private_references = $this-> input-> post('Private_reference'); 
			$Resipient = $this-> input-> post('Resipient'); 
			$Address = $this-> input-> post('Address'); 
			$City = $this-> input-> post('City'); 
			$Phone = $this-> input-> post('Phone'); 
			$Order_status = $this-> input-> post('Order_status'); 
			$Products = $this-> input-> post('Product_id'); 
			
			foreach ($Private_references as $key => $Private_reference) {
				$city = $this-> Shipping_model-> get_city_by_name($City[$key]);
				//generating and updating search code
	 			$search_code = $this-> generate_order_search_code($franchise);
				$this-> update_order_search_code($franchise);
				//getting product details
				$product = $this-> Product_model-> get_franchises_products($franchise,$Products[$key]);
				//getting shipping info
				$shipping_info = $this->Shipping_model-> get_shipping_info($franchise,$city['id']);

				$franchise_shipping_cost = $this->Shipping_model-> get_shipping_cost($product->weight,$shipping_info->franchise_shipping_cost,$shipping_info->franchise_charge_for_additional_kg);
				$service_charge = $this->Shipping_model-> get_shipping_cost($product->weight,$shipping_info->service_charge,$shipping_info->charge_for_additional_kg);

				$data = array(
					'search_code' => $search_code,
					'franchise' => $franchise,
					'resipient'=> $Resipient[$key],
					'private_reference'=> $Private_references[$key],
					'address'=> $Address[$key],
					'city'=> $city['id'],
					'phone_one'=> $Phone[$key], 
					'payment_type'=> 'COD',
					'created_by'=> $user_id,
					'order_status'=> $Order_status[$key],
					'last_modified'=> date('Y-m-d H:i:s'),
					'total_amount'=> $product-> price,
					'total_weight'=> $product-> weight,
					'total_commision'=> $product-> commision,
					'item_description'=> $product-> short_title."-1",
					'franchise_shipping_cost'=> $franchise_shipping_cost,
					'service_charge'=> $service_charge,
					'grand_total'=> $product-> price + $franchise_shipping_cost, 			
				);

				$this-> db-> insert('order',$data);
				$orderID = $this->db->insert_id();

				if($product->product_type=='product_group'){
				    $child_products = explode (",", $product-> child_products);
				    $total_amount = 0; $item_description = '';
				    foreach($child_products as $item){
				        $child_product = $this-> Product_model-> get_franchises_products($franchise,$item);
				        $data = array(
        					'order'=> $orderID,
        					'product'=> $child_product->id,
        					'quantity'=> 1,
        					'amount'=> $child_product-> price,
        					'commision'=> $child_product-> commision			
        				);
        				//inserting data to order items 
    				    $this-> db-> insert('order_items',$data);
    				    $total_amount+=$child_product-> price;
    				    $item_description.= $child_product-> short_title."-1| ";
				    }
				    $item_description = rtrim($item_description,"| ");
				    $discount = ($product-> price + $franchise_shipping_cost)- $total_amount;
				    $order_data = array(
				        'item_description'=>$item_description,
        				'franchise_shipping_cost'=> $discount			
        			);
        
        			$this-> db-> where('id',$orderID);
        			$this-> db-> update('order',$order_data);
				    
				}else{
				    //data for order_items table
    				$data = array(
    					'order'=> $orderID,
    					'product'=> $product->id,
    					'quantity'=> 1,
    					'amount'=> $product-> price,
    					'commision'=> $product-> commision			
    				);
    				//inserting data to order items 
    				$this-> db-> insert('order_items',$data);
				}
				
			}
			$this->db->trans_complete();
		}
		
		public function dispatch_order($order_id){
		    $user_id = $this-> session-> userdata('user_id');
		    $order_data = array(
				'order_status'=> 'in_courier',
				'dispatched_date'=> date("Y-m-d")		
			);

			$this-> db-> where('id',$order_id);
			$this-> db-> update('order',$order_data);

			$this-> Inventory_model-> dispatch_stock_by_order($order_id); 
			
			//updating order modification history
	        $this-> Order_model-> update_order_modification_history($order_id,$user_id,'Order dispatched.');
		}
		
		
		public function update_only_order_status($order,$status){

    		$this-> db-> where('id',$order);
    		$this-> db-> update('order',array('order_status'=> $status));
    		return $order;
    
    	}


//*****************selecting methods**************************************	

		public function get_order($search_code){
			$query = $this-> db-> get_where('order',array('search_code'=> $search_code));
			return $query-> row();
		}

		public function get_order_field($order,$field){
			$this-> db-> select($field);
			$query = $this-> db-> get_where('order',array('id'=> $order));
			return $query-> row()-> $field;
		}

		public function get_order_field_by_key($key,$key_field,$field=FALSE){
			if ($field) {
				$this-> db-> select($field);
				$query = $this-> db-> get_where('order',array($key_field=> $key));
				return $query-> row()-> $field;	
			}
			$query = $this-> db-> get_where('order',array($key_field=> $key));
			return $query-> row();
		}

		public function get_order_field_by_search_code($search_code,$field){
			$this-> db-> select($field);
			$query = $this-> db-> get_where('order',array('search_code'=> $search_code));
			return $query-> row()-> $field;
		}

		public function get_orders($where = array(),$filter_users_orders = false){
		    if ($filter_users_orders==true) {
				$user_id = $this-> session-> userdata('user_id');
				$franchises = $this-> Franchise_model-> get_users_franchise_ids($user_id);
				$this->db->where_in('order.franchise', $franchises);
			}
			if (!empty($where)) {
				$this->db->where($where);
			}
			//update
			if($this-> session-> userdata('user_role')=='shop_admin'){
			    $this->db->where('order.created_by', $this-> session-> userdata('user_id'));
			}
			$this-> db-> select('order.id,search_code,resipient,order_status,order.phone_one,order.phone_two,order.item_description,order.call_attempts,order.grand_total,order.customer_notes,order.delivery_notes,order.last_modified,hold_orders.hold_until,hold_orders.reason,order.address,order.dispatched_date,city.city,city.district,city.province,city.zip_code,payment_type,total_weight,franchise.name as franchise,courier.name as courier,courier.tracking_url,order.created_date,order.courier_reference,order.private_reference,order.return_informed,user.name as created_by');
			$this-> db-> join('courier', 'order.courier = courier.id','left');
			$this-> db-> join('franchise','order.franchise = franchise.id');
			$this-> db-> join('hold_orders', 'order.id = hold_orders.order','left');
			$this-> db-> join('city', 'order.city = city.id','left');
			$this-> db-> join('user', 'order.created_by = user.id');
			$this->db->order_by('order.search_code', 'ASC');
			$query = $this-> db-> get('order');
			return $query-> result_array();
		}

		public function get_call_list($order_status,$limit,$offset){
		    $shop_admins = $this-> User_model-> get_shop_admins();
		    $this-> db-> limit($limit, $offset);
		    if($order_status=='want_info'){
		        $this->db->where('order_status', 'not_confirm'); 
		        $this->db->where('hold_orders.reason', 'Want Information'); 
		    }else{
		        $this->db->where('order_status', $order_status);   
		    }
			
			$this->db->where('order.franchise', $this-> session-> userdata('franchise'));

			if($this-> session-> userdata('user_role')=='shop_admin'){
			    $this->db->where('order.created_by', $this-> session-> userdata('user_id'));
			}else{
			    $shop_admins = array_column($shop_admins,"id");
			    $this->db->where_not_in('order.created_by', $shop_admins);
			}

			$this-> db-> select('order.id,search_code,resipient,order_status,phone_one,phone_two,item_description,call_attempts,grand_total,customer_notes,delivery_notes,last_modified,hold_until,reason,address,city.city,city.zip_code,payment_type,total_weight,order.created_date,order.private_reference');
			$this-> db-> join('hold_orders', 'order.id = hold_orders.order','left');
			$this-> db-> join('city', 'order.city = city.id','left');
			$this->db->order_by('order.call_attempts', 'ASC');
			$this->db->order_by('order.created_date', 'DESC');
			$query = $this-> db-> get('order');
			$calllist = $query-> result_array();
			
			foreach($calllist as $key=>$item){
				$calllist[$key]['order_items'] = $this->get_order_items($calllist[$key]['id']); //edited
			    if($this->check_phone_number_exists($item['search_code'],$item['phone_one'])){
			        $calllist[$key]['phone_one_exists'] = true;
			    }else{
			        $calllist[$key]['phone_one_exists'] = false;
			    }
			    
			    if($item['phone_two']==null || $item['phone_two']==''){
			        $calllist[$key]['phone_two_exists'] = false;
			    }else{
			        if($this->check_phone_number_exists($item['search_code'],$item['phone_two'])){
    			        $calllist[$key]['phone_two_exists'] = true;
    			    }else{
    			        $calllist[$key]['phone_two_exists'] = false;
    			    }
			    }
			}
			
			return $calllist;
			
		}
		
		public function get_call_list_count($order_status){
		    $shop_admins = $this-> User_model-> get_shop_admins();
			$this->db->where('order.franchise', $this-> session-> userdata('franchise'));

			if($this-> session-> userdata('user_role')=='shop_admin'){
			    $this->db->where('order.created_by', $this-> session-> userdata('user_id'));
			}else{
			    $shop_admins = array_column($shop_admins,"id");
			    $this->db->where_not_in('order.created_by', $shop_admins);
			}
			
			if($order_status=='want_info'){
		        $this->db->where('order_status', 'not_confirm'); 
		        $this->db->where('hold_orders.reason', 'Want Information'); 
		        $this-> db-> join('hold_orders', 'order.id = hold_orders.order','left');
		    }else{
		        $this->db->where('order_status', $order_status);   
		    }
			$this->db->from('order');
			return $this->db->count_all_results();
		}
		
		public function check_phone_number_exists($search_code,$phone_number){
		    
		    $this-> db-> where('created_date>=',date("Y-m-d", strtotime( '-30 days' )));
		    $this-> db-> where('search_code<>',$search_code);
		    $this->db->group_start();
    		    $this-> db-> where('phone_one',$phone_number);
    		    $this-> db-> or_where('phone_two',$phone_number);
		    $this->db->group_end();
		    $query = $this-> db-> get('order');
		    
		    if( ! empty($query-> result_array())){
		        return true;
		    }else{
		        return false;
		    }
		}
		
		public function get_order_modification_history($search_code){
		    $this-> db-> select('order.search_code,order.id,order.order_status,order.resipient,address,order.phone_one,order.phone_two,city.city,order.created_date,order.item_description,courier_reference,courier.name as courier,order.damaged_return,order.is_urgent');
		    $this-> db-> join('city', 'order.city = city.id');
		    $this-> db-> join('courier', 'order.courier = courier.id','left');
		    $query = $this-> db-> get_where('order',array('order.search_code'=> $search_code));
		    $order_data = $query-> row_array();
		    
		    $this-> db-> select('order_modification_history.modified_on,order_modification_history.remarks,user.name as user');
			$this-> db-> join('user', 'order_modification_history.user = user.id');
			$query = $this-> db-> get_where('order_modification_history',array('order'=> $order_data['id']));
			$order_data['order_history'] = $query-> result_array();
			return $order_data;
		}

		public function get_order_items($order){
			$this-> db-> select('order_items.product,quantity,price,amount,product.short_title,product.title,product.stock');//edited
			$this-> db-> join('product', 'product.id = order_items.product');
			$query = $this-> db-> get_where('order_items',array('order'=> $order));
			return $query-> result_array();
		}
		
		public function get_order_items_for_print($order){
			$this-> db-> select('order_items.product,quantity,franchise_product_map.price,order_items.amount,product.short_title,product.title');
			$this-> db-> join('product', 'product.id = order_items.product');
			$this-> db-> join('order', 'order.id = order_items.order');
			$this-> db-> join('franchise_product_map', 'franchise_product_map.product = order_items.product AND franchise_product_map.franchise = order.franchise','left');
			$query = $this-> db-> get_where('order_items',array('order'=> $order));
			return $query-> result_array();
		}

		public function get_users_allowed_status($user){
			$this->db->order_by('order_status.priority', 'ASC');
			$this-> db-> join('allowed_status', 'allowed_status.order_status = order_status.id');
			$query = $this-> db-> get_where('order_status',array('user'=> $user));
			return $query-> result_array();			
		}

		public function generate_order_search_code($franchise){
			$query = $this-> db-> get_where('search_code',array('franchise'=> $franchise));
			$result= $query-> row();
			$search_code = $result-> order_prefix."-".($result-> last_order + 1); 
			return $search_code;
		}

		public function check_status_allowed($status, $user){
			$query = $this-> db-> get_where('allowed_status', array('user'=> $user, 'order_status' => $status));
			if (empty($query->row_array())) {
				return false;
			} else {
				return true;
			}	
		}

		public function is_user_allowed_franchise($franchise, $user){
			$query = $this-> db-> get_where('user_franchise_map', array('user'=> $user, 'franchise' => $franchise));
			if (empty($query->row_array())) {
				return false;
			} else {
				return true;
			}	
		}

		public function get_call_attempts($order){
			$this-> db-> select('call_attempts');
			$query = $this-> db-> get_where('order',array('id'=> $order));
			return $query-> row()-> call_attempts;
		}

		public function is_ready_for_call($order){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('id'=> $order));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='pending' || $result['order_status']=='confirm' || $result['order_status']=='hold' || $result['order_status']=='not_confirm') {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}

		public function is_ready_for_packing($search_code){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('search_code'=> $search_code));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='confirm' || $result['order_status']=='not_confirm' || $result['order_status']=='pending') {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}

		public function is_ready_for_pending_return($courier_ref){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('courier_reference'=> $courier_ref));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='in_courier') {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}

		public function is_ready_for_settlement($courier_ref){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('courier_reference'=> $courier_ref));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='in_courier'|| $result['order_status']=='pending_return' || $result['order_status']=='returned' || $result['order_s	tatus']=='delivered' || ($result['order_status']=='settled' && $result['courier']=='7')) {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}

		public function is_ready_for_return($courier_reference){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('courier_reference'=> $courier_reference));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='in_courier' || $result['order_status']=='pending_return' || ($result['order_status']=='settled' && $result['courier']=='7')) {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}
		
		public function is_ready_for_dispatch($courier_reference){
			//checks whether an order exists by search code and it is confirmed  
			$query = $this-> db-> get_where('order', array('courier_reference'=> $courier_reference));
			$result = $query->row_array();
			if (!empty($result)) {
				if ($result['order_status']=='in_packing') {
					return true;	
				}else{
					return false;
				}
			} else {
				return false;
			}	
		}
		
		public function get_order_meta($meta_type){
			$this->db->order_by('meta_priority', 'ASC');
			$query = $this-> db-> get_where('additional_order_info_meta', array('meta_type'=> $meta_type));
			return $query-> result_array();
		}

		public function get_order_additional_info($meta_type,$order_id){
			$this-> db-> join('additional_order_info_meta', 'additional_order_info.order_meta = additional_order_info_meta.meta_key');
			$this-> db-> where('meta_type',$meta_type);
			$this-> db-> where('order_id',$order_id);
			$this->db->order_by('meta_priority', 'ASC');
			$query = $this-> db-> get('additional_order_info');
			
			return $query-> result_array();
		}

		public function get_order_with_additional_info($meta_type){
			$this->db->distinct();
			$this-> db-> select('order.search_code,order.id,order.resipient,order.phone_one,order.phone_two,order.item_description,courier_reference');
			$this-> db-> join('additional_order_info', 'additional_order_info.order_id = order.id');
		//	$this->db->order_by('order.created_date', 'DESC');
			$query = $this-> db-> get('order');
			$order_data = $query-> result_array();

			foreach ($order_data as $key => $item) {
				$additional_data = $this-> get_order_additional_info($meta_type,$item['id']);

				foreach ($additional_data as $item) {
					if ($item['input_type']=='checkbox') {
						if ($item['value']=='on') {
							$order_data[$key][$item['meta_key']] = 'Yes';
						}else{
							$order_data[$key][$item['meta_key']] = 'No';
						}
					}else{
						$order_data[$key][$item['meta_key']] = $item['value'];
					}
				}
			}	
			return $order_data;	
		}
		
		public function get_order_complaints($search_code){
		    $this-> db-> select('order.search_code,order.id,order.resipient,address,order.phone_one,order.phone_two,city.city,order.created_date,order.item_description,courier_reference,courier.name as courier');
		    $this-> db-> join('city', 'order.city = city.id');
		    $this-> db-> join('courier', 'order.courier = courier.id','left');
		    $query = $this-> db-> get_where('order',array('order.search_code'=> $search_code));
		    $order_data = $query-> row_array();
		    
		    $this-> db-> select('complaint.id,complaint.complaint_status,complaint.remark,created.name as created_by,complaint.created_date,checked.name as checked_by,complaint.checked_date,complaint.solution');
			$this-> db-> join('user created','complaint.created_by = created.id','left');
			$this-> db-> join('user checked','complaint.checked_by = checked.id','left');
			$query = $this-> db-> get_where('complaint',array('order'=> $order_data['id']));
			$order_data['complaints'] = $query-> result_array();
			return $order_data;
		}

		public function get_complaint($id=NULL){
			if ($id==NULL) {
				$this-> db-> select('order.search_code,order.id,order.resipient,order.address,order.phone_one,order.phone_two,city.city,order.created_date,order.item_description,courier_reference,courier.name as courier,complaint.remark,complaint.created_date as complaint_date,complaint.complaint_status,created.name as created_by,checked.name as checked_by,complaint.checked_date,complaint.solution');
		    	$this-> db-> join('order', 'order.id = complaint.order');
		    	$this-> db-> join('city', 'order.city = city.id');
		    	$this-> db-> join('courier', 'order.courier = courier.id','left');
		    	$this-> db-> join('user created','complaint.created_by = created.id','left');
				$this-> db-> join('user checked','complaint.checked_by = checked.id','left');

		    	$query = $this-> db-> get_where('complaint',array('order.franchise'=>$this-> session-> userdata('franchise')));
		    	return $query-> result_array();
			}
			$query = $this-> db-> get_where('complaint',array('id'=> $id));
			return $query-> row();
		}
		
	public function get_packing_batch_order($batch_code){
		$this->db->select('packing_batch_items.packing_batch, order.search_code, order.id, order.order_status');
		$this->db->join('order', 'packing_batch_items.order = order.id');
		$query = $this-> db-> get_where('packing_batch_items',array('packing_batch_items.packing_batch'=>$batch_code));
		return $query->result_array();
	}


	}	